'use strict';

const uuid = require('uuid');
const AWS = require('aws-sdk');
const isValidJson = require('../utils/validation/jsonValidation');
const {getResponse, getErrorResponse} = require('../utils/responses');
const {writeObservationRoundToDb, writeObservationsToDb} = require('../db/database');
const {getMissingParameters, getMissingParametersErrorMessage} = require('../utils/validation/observationRoundValidation');

const BUCKET_NAME = 'sheep-image-storage';
const s3 = new AWS.S3({
    signatureVersion: 'v4',
});

function getImage(base64String) {
    return new Buffer(base64String.replace(/^data:image\/\w+;base64,/, ""), 'base64');
}

function getObservations(registrationPoints) {
    let observations = [];

    const keys = Object.keys(registrationPoints);

    for (let i = 0; i < keys.length; i++) {
        const k = keys[i];
        const registrations = registrationPoints[k].observations;
        const observationDataList = Object.keys(registrations).map(element => {
            const item = registrations[element];
            item.appId = item.id;
            item.id = uuid.v1();

            return {
                PutRequest: {
                    Item: item
                }
            };
        });
        observations = observations.concat(observationDataList);
    }
    return observations;
}

function getObservationRoundObject(data) {
    const timestamp = new Date().getTime();
    const round = {
        id: uuid.v1(),
        dbCreatedAt: timestamp,
        dbUpdatedAt: timestamp,
        appId: data.id,
        farmNumber: data.farmNumber,
        buildingNumber: data.buildingNumber,
        municipality: data.municipality,
        description: data.description,
        name: data.name,
        numberOfParticipants: data.numberOfParticipants,
        weather: data.weather,
        createdTimestamp: data.createdTimestamp,
        lastSavedTimestamp: data.lastSavedTimestamp,
        homePosition: data.homePosition,
        userTrackingPoints: data.userTrackingPoints,
        registrationPoints: data.registrationPoints
    };

    // Remove
    const registrationPoints = round.registrationPoints;

    const res = Object.keys(registrationPoints).map(point => {
        const observations = Object.keys(registrationPoints[point].observations).map(observationKey => {
            return registrationPoints[point].observations[observationKey];
        });
        const ssss = registrationPoints[point];
        ssss.observations = observations;
        return ssss;
    });

    const ja = round;
    ja.registrationPoints = res;
    return ja;

}


function upload(image) {


    return new Promise((resolve, reject) => {
        const filename = uuid.v1() + '.jpg';

        s3.putObject({
            Bucket: BUCKET_NAME,
            Key: filename,
            ContentEncoding: 'base64',
            ContentType: 'image/jpeg',
            Body: getImage(image.base64)
        }, (result) => {
            if (result) {
                reject('An error occurred while uploading image.');
            } else {
                resolve('https://s3.eu-central-1.amazonaws.com/sheep-image-storage/' + filename);
            }
        });
    });
}

function uploadImages(images) {
    return new Promise((resolve, reject) => {
        const promises = [];

        for (let i = 0; i < images.length; i++) {
            promises.push(upload(images[i]));
        }
        Promise.all(promises)
            .then(res => {
                resolve(res);
            }).catch(error => {
                reject('An error occurred while uploading image.');
            });
    });
}


module.exports = (event, context, callback) => {
    // Validate JSON
    if (!isValidJson(event.body)) {
        callback(null, getErrorResponse('Invalid JSON', 400));
        return;
    }
    const data = JSON.parse(event.body);

    // Check that required fields are present in body
    const dataKeys = [
        'farmNumber',
        'buildingNumber',
        'municipality',
        'name',
        'description',
        'id',
        'numberOfParticipants',
        'weather',
        'userTrackingPoints',
        'registrationPoints',
        'homePosition',
        'createdTimestamp',
        'lastSavedTimestamp',
    ];

    const missingKeys = getMissingParameters(data, dataKeys);
    if (missingKeys.length > 0) {
        const errorMessage = getMissingParametersErrorMessage(missingKeys);
        callback(null, getErrorResponse(errorMessage, 400));
        return;
    }

    const observationRound = getObservationRoundObject(data);
    ////////
    const regPointIds = Object.keys(observationRound.registrationPoints);

    var res = observationRound; //{...observationRound};

    const promises = [];

    for (let i = 0; i < regPointIds.length; i++) {
        const observations = observationRound.registrationPoints[regPointIds[i]].observations;
        const observationIds = Object.keys(observations);

        for (let j = 0; j < observationIds.length; j++) {
            const images = observations[observationIds[j]].data.images;
            const p = new Promise((resolve, reject) => {
                uploadImages(images).then(img => {
                    res.registrationPoints[regPointIds[i]].observations[observationIds[j]].data.images = img;
                    resolve();
                }).catch(error => {
                    reject(error);
                })
            });
            promises.push(p);
        }

    }

    Promise.all(promises)
        .then(result => {
            // Write to DB
            writeObservationRoundToDb(res).then(() => {
                const observations = getObservations(data.registrationPoints);
                const successResponse = getResponse(observationRound);
                if (observations.length > 0) {
                    writeObservationsToDb(observations).then(() => {
                        callback(null, successResponse);
                    }).catch(() => {
                        callback(null, getErrorResponse('Failed to store observations in db'))
                    });
                } else {
                    callback(null, successResponse);
                }
            }).catch(() => {
                callback(null, getErrorResponse('Failed to store observation round in db!'));
            });

            // callback(null, getResponse(res))
        }).catch(error => {
            callback(null, getErrorResponse('Something went wrong....' + JSON.stringify(error)))
        });
};
