import React, { Component } from 'react';
import StatisticContainer from '../../common/StatisticContainer.jsx';
import { getDateAsString, getTimeAsString } from '../../utilities/dateParser';
import { getNumberOfSheepInObservationRound, getWalkedDistance, getTimeSpent } from '../../utilities/statistics';
import { getSettings } from '../../utilities/save.js';
import { hasConnection } from '../../utilities/connectionDetection.js';
import { uploadRoundData } from '../../utilities/apiCommunication.js';
import { openWarningWindow, openSuccessWindow } from '../../utilities/warning_window';
import * as transition from '../../utilities/transitions';

class InfoPage extends Component {

  constructor(props) {
    super(props);

    const state = this.props.state;
    const numberObject = getNumberOfSheepInObservationRound(state);

    this.state = {
      numberOfSheep: numberObject.lamb + numberObject.ewes,
      distanceWalked: getWalkedDistance(state),
      roundTime: getTimeSpent(state),
      hasBeenUploaded: state.hasBeenUploaded,
      startDate: getDateAsString(state.createdTimestamp),
      startTime: getTimeAsString(state.createdTimestamp),
      farmNumber: state.farmNumber,
      buildingNumber: state.buildingNumber,
      municipality: state.municipality,
      roundIsComplete: state.roundIsComplete,
      hasConnection: hasConnection(),
      showConnectionText: false
    };

    this.onLoad = this.onLoad.bind(this);
    this.uploadData = this.uploadData.bind(this);
    this.setConnection = this.setConnection.bind(this);
    this.showNoConnectionText = this.showNoConnectionText.bind(this);
    this.onOffline = this.onOffline.bind(this);
    this.onOnline = this.onOnline.bind(this);
    this.onUploadSuccess = this.onUploadSuccess.bind(this);
  }

  componentWillMount() {
    if (getSettings().automaticUploadIsEnabled) {
      document.addEventListener('online', this.uploadData, false);
    } else {
      document.addEventListener('online', this.onOnline, false);
      document.addEventListener('offline', this.onOffline, false);
    }
  }

  componentDidMount() {
    transition.animateRightToLeft('observation-information');
  }

  componentWillUnmount() {
    if (getSettings().automaticUploadIsEnabled) {
      document.removeEventListener('online', this.uploadData, false);
    } else {
      document.removeEventListener('online', this.onOnline, false);
      document.removeEventListener('offline', this.onOffline, false);
    }
  }

  onOffline() {
    this.setState({
      hasConnection: false
    });
  }

  onOnline() {
    this.setState({
      hasConnection: true,
      showConnectionText: false
    });
  }

  setConnection() {
    this.setState({
      hasConnection: hasConnection()
    });
  }

  showNoConnectionText() {
    this.setState({
      showConnectionText: true
    });
  }

  onUploadSuccess() {
    this.props.roundHasBeenUploaded();
    this.setState({
      hasBeenUploaded: true
    });
    openSuccessWindow('Dataene har blitt lastet opp til server!');
  }

  uploadData() {
    const data = this.props.state;
    const dataIsInvalid = data === null || data === undefined;
    if (!dataIsInvalid) {
      uploadRoundData(data, this.onUploadSuccess, () => openWarningWindow('Dataene kunne ikke lastes opp. Prøv på nytt på et senere tidspunkt!'));
    }
  }

  onLoad(stateToLoad) {
    this.props.onLoad(stateToLoad);
  }

  render() {
    const { automaticUploadIsEnabled } = getSettings();

    return (
      <div id="observation-information" className="load-observation-information">
        <div className="active-round-info-container" >
          <div className="statistics">
            <StatisticContainer
              value={this.state.numberOfSheep}
              imagePath="img/vectors/NewObservationForm/sheep_with_lamb_white.svg"
            />
            <StatisticContainer
              value={this.state.distanceWalked}
              iconClass="fa fa-blind"
            />
            <StatisticContainer
              value={this.state.roundTime}
              iconClass="fa fa-clock-o"
            />
          </div>
          <div className="round-metadata">
            <div className="data-row">
              <div className="column">
                Startdato:
              </div>
              <div className="column">
                {this.state.startDate}
              </div>
            </div>
            <div className="data-row">
              <div className="column">
                Starttid:
              </div>
              <div className="column">
                {this.state.startTime}
              </div>
            </div>
            <div className="data-row">
              <div className="column">
                Kommune:
              </div>
              <div className="column">
                {this.state.municipality}
              </div>
            </div>
            <div className="data-row">
              <div className="column">
                Gårdsnummer:
              </div>
              <div className="column">
                {this.state.farmNumber}
              </div>
            </div>
            <div className="data-row">
              <div className="column">
                Rundestatus:
              </div>
              <div className="column">
                {this.state.roundIsComplete ? 'Fullført' : 'Aktiv'}
              </div>
            </div>

            <div className={`data-row${this.state.roundIsComplete ? '' : ' hide'}`}>
              <div className="column">
                Status opplasting:
              </div>
              <div className="column">
                <i className={`fa fa-2x upload-status-icon${this.state.hasBeenUploaded ? ' fa-check' : ' fa-times'}`} />
              </div>
            </div>
            <div className={`upload-disclaimer-text${this.state.roundIsComplete && !this.state.hasBeenUploaded && automaticUploadIsEnabled ? ' show' : ''}`}>
              <i className="fa fa-info-circle" />
              Opplasting skjer automatisk ved tilkobling til internett.
            </div>
          </div>
          <div className={`load-button-container${this.state.roundIsComplete ? '' : ' show'}`}>
            <button
              onClick={() => this.onLoad(this.props.state)}
              className="load-observation-button"
            >
              <div>
                Last inn runde
              </div>
              <i className="fa fa-angle-right" />
            </button>
          </div>
          <div
            className={`load-button-container
                ${this.state.roundIsComplete && !this.state.hasBeenUploaded && !automaticUploadIsEnabled ? ' show' : ''}`}
          >
            <div className="no-connection-text-container">
              <label className={`no-connection-text${this.state.showConnectionText ? ' show' : ''}`}>
                <i className="fa fa-info-circle" /> Koble til internett for å laste opp
              </label>
            </div>
            <button
              onClick={this.state.hasConnection ? this.uploadData : this.showNoConnectionText}
              className={`load-observation-button${this.state.hasConnection ? '' : ' disabled'}`}
            >
              <div>
                Last opp
              </div>
              <i className="fa fa-cloud-upload" />
            </button>
          </div>
        </div>
      </div>
    );
  }
}

export default InfoPage;
