import L from 'leaflet';
import React, { Component } from 'react';
import { getMarkerIcon } from './mapMarker';
import {
  Circle,
  CircleMarker,
  Map,
  Marker,
  Polygon,
  Polyline,
  Popup,
  Rectangle,
  TileLayer } from 'react-leaflet'
import image from '../style/img/vectors/sheep-small-icon.svg';

const markerSettings = {
  iconSize: [33, 56],
  iconAnchor: [16.5, 56],
  popupAnchor: [-3, -76]
};

export const mapUrl = 'http://opencache.statkart.no/gatekeeper/gk/gk.open_gmaps?layers=topo4&zoom={z}&x={x}&y={y}';

export const mapStyle = {
  height: '100%',
  width: '100%'
};

export function getMarker(position, icon) {
  return <Marker position={position} icon={getIcon()} />
}

export function getIcon() {
  return new L.divIcon({
    html: '<div class="marker-container">' +
    '<div class="marker-icon">' +
    '<img src="' + image + '" class="sheep-observation-marker" />' +
    '</div>' +
    '<div class="marker-line"></div>' +
    '<div>',
    ...markerSettings
  });
}

export function getStateByObservation(observationRound) {
  if (observationRound === null || observationRound === undefined) {
    return {
      homePosition: [63.426739, 10.382404],
      userPolyline: [],
      registrationPoints: {}
    }

  } else {
    const homePosition = [observationRound.homePosition.latitude, observationRound.homePosition.longitude];
    return {
      homePosition,
      userPolyline: observationRound.userTrackingPoints,
      registrationPoints: observationRound.registrationPoints
    }
  }
}

function drawStandard(mapSettings, round) {
  let mapGraphics = [];
  if (mapSettings.showUserTrackingLine) {
    mapGraphics.push(<Polyline key={'line' + round.id} positions={round.userTrackingPoints} color="red" dashArray="7,10"  />);
    const homePosition = [round.homePosition.latitude, round.homePosition.longitude];
    mapGraphics.push(<Marker key={'home' + round.id} position={homePosition} icon={getMarkerIcon({type: 'home'})}/>);
  }

  if (mapSettings.showRegistrationPoints || mapSettings.showObservationPoints) {
    const registrationPoints = round.registrationPoints;
    Object.keys(registrationPoints).map(key => {
      const registrationPoint = registrationPoints[key];
      const registrationPosition = [registrationPoint.latitude, registrationPoint.longitude];
      let observationPosition = [];
      if (mapSettings.showObservationPoints) {
        Object.keys(registrationPoint.observations).forEach((key, index) => {
          const observation = registrationPoint.observations[key];
          observationPosition = [observation.latitude, observation.longitude];
          mapGraphics.push(<Marker key={key} position={observationPosition} icon={getMarkerIcon(observation.data)}/>);
          if (mapSettings.showRegistrationPoints) {
            mapGraphics.push(<Polyline key={'poly' + key + index}  positions={[registrationPosition, observationPosition]} />);
          }
        });
      }
      if (mapSettings.showRegistrationPoints) {
        mapGraphics.push(<Marker key={key} position={registrationPosition} icon={getMarkerIcon({type: 'registration'})}/>);
      }
    });
    return mapGraphics;
  }
  return null;
}

function getNumberOfSheepInObservation(data) {
  const ewes = {
    white: 0,
    black: 0,
    blackAndWhite: 0
  };
  const lamb = {
    white: 0,
    black: 0,
    blackAndWhite: 0
  };
  data.groups.forEach(groups => {
    ewes.white += groups.ewes.white;
    ewes.black += groups.ewes.black;
    ewes.blackAndWhite += groups.ewes.blackAndWhite;
    lamb.white += groups.lamb.white;
    lamb.black += groups.lamb.black;
    lamb.blackAndWhite += groups.lamb.blackAndWhite;
  });
  return {
    ewes: {
      ...ewes,
      total: ewes.black + ewes.white + ewes.blackAndWhite
    },
    lamb: {
      ...lamb,
      total: lamb.black + lamb.white + lamb.blackAndWhite
    }
  };
}

function getIntensityObservation(observation, mapSettings, key) {
  const observationPosition = [observation.latitude, observation.longitude];

  switch (observation.data.type) {
    case 'Sau': {
      if (mapSettings.showSheep) {
        const numberOfSheep = getNumberOfSheepInObservation(observation.data);
        const total = numberOfSheep.ewes.total + numberOfSheep.lamb.total;
        return (
            <Circle key={'circle' + key} center={observationPosition} color="red" fillColor="red" radius={total * 2} opacity="0.4" fillOpacity="0.4" />
        )
      }
      break;
    }
    case 'Død sau': {
      if (mapSettings.showDeadSheep) {
        return <Circle key={'circle' + key} center={observationPosition} color="blue" fillColor="blue" radius={2} opacity="1" fillOpacity="1" />;
      }
      break;
    }
    case 'Ullfunn': {
      if (mapSettings.showWoolFindings) {
        return <Circle key={'circle' + key} center={observationPosition} color="blue" fillColor="blue" radius={2} opacity="1" fillOpacity="1" />;
      }
      break;
    }
    case 'Hund': {
      if (mapSettings.showDogs) {
        return <Circle key={'circle' + key} center={observationPosition} color="blue" fillColor="blue" radius={2} opacity="1" fillOpacity="1" />;
      }
      break;
    }
    case 'Rovdyr': {
      if (mapSettings.showPredators) {
        return <Circle key={'circle' + key} center={observationPosition} color="blue" fillColor="blue" radius={2} opacity="1" fillOpacity="1" />;
      }
      break;
    }
    case 'Annet':
    case 'Reinsdyr': {
      if (mapSettings.showOther) {
        return <Circle key={'circle' + key} center={observationPosition} color="blue" fillColor="blue" radius={2} opacity="1" fillOpacity="1" />;
      }
      break;
    }

    default:
      return null;
  }
}

function drawIntensity(round, mapSettings) {
  let mapGraphics = [];
  const registrationPoints = round.registrationPoints;
  Object.keys(registrationPoints).map(key => {
    const registrationPoint = registrationPoints[key];
    Object.keys(registrationPoint.observations).forEach(key => {
      const observation = registrationPoint.observations[key];
      mapGraphics.push(getIntensityObservation(observation, mapSettings, key));
    });
  });
  return mapGraphics;
}

export function drawRegistrationPointsAndObservations(mapSettings, round = {}, displayType = 'standard') {
  switch (displayType) {
    case 'standard': {
      return drawStandard(mapSettings, round);
    }
    case 'intensity': {
      return drawIntensity(round, mapSettings);
    }
    default:
      return null;
  }

}
