import React from 'react';
import {
  getNumberOfSheepInObservation,
  getNumberOfSheepInObservationRound,
  getWalkedDistance,
  getTimeSpent,
  getRoundStatistics,
  getSheepColor
} from './statistics';
import { getDateAsString } from './dateParser';
import StatHolder from '../common/StatHolder';
import DataRow from '../common/MetadataRow';
import { latLngBounds } from 'leaflet';
import jspdf from 'jspdf';
import Map from '../common/SingleObservationMap';
import LambImage from '../style/img/vectors/NewObservationForm/lamb_white.svg';
import EweImage from '../style/img/vectors/NewObservationForm/ewe_white.svg';
import SheepImage from '../style/img/vectors/NewObservationForm/sheep_with_lamb_white.svg';
import DogImage from '../style/img/vectors/NewObservationForm/dog_white.svg';
import WoolImage from '../style/img/vectors/NewObservationForm/wool_white.svg';
import PredatorImage from '../style/img/vectors/NewObservationForm/predator_main_white.svg';
import DeadImage from '../style/img/vectors/NewObservationForm/dead_sheep_white.svg';
import InjuredImage from '../style/img/vectors/NewObservationForm/injured_sheep_white.svg';
import ReindeerImage from '../style/img/vectors/NewObservationForm/reindeer_white.svg';
import OtherImage from '../style/img/vectors/NewObservationForm/unknown_white.svg';

export function compare(a, b) {
  if (a.createdTimestamp < b.createdTimestamp)
    return -1;
  if (a.createdTimestamp > b.createdTimestamp)
    return 1;
  return 0;
}

export function saveDocumentAsPdf(){
  let pdf = new jspdf('p', 'pt', 'letter');
  let source = document.getElementById('report-pages');
  let specialElementHandlers = {
    '#bypassme': (element, renderer) => {
      return true
    }
  };

  let margins = {
    top: 70,
    left: 70,
    width: 465
  };

  pdf.fromHTML (
      source, // HTML string or DOM elem ref.
      margins.left, // x coord
      margins.top, // y coord
      {
        width: margins.width, // max width of content on PDF
        elementHandlers: specialElementHandlers
      },
      dispose => {
        // dispose: object with X, Y of the last line add to the PDF
        // this allow the insertion of new lines after html
        pdf.save('tilsynsrapport.pdf');
      }
  )
}

export function getMapBoundsForEntireRound(round) {
  const homePosition = [round.homePosition.latitude, round.homePosition.longitude];
  const bounds = latLngBounds([homePosition, homePosition]);
  if (round.registrationPoints !== null && round.registrationPoints !== undefined) {
    Object.keys(round.registrationPoints).forEach(key => {
      const registrationPoint = round.registrationPoints[key];
      bounds.extend([registrationPoint.latitude, registrationPoint.longitude]);
      Object.keys(registrationPoint.observations).forEach(key => {
        const observation = registrationPoint.observations[key];
        bounds.extend([observation.latitude, observation.longitude]);
      })
    });
  }
  return bounds;
}

export function getBoundsForObservation(registrationPoint, observation) {
  const regPosition = [registrationPoint.latitude, registrationPoint.longitude];
  const observationPosition = [observation.latitude, observation.longitude];
  return [regPosition, observationPosition];
}

export function observationRoundBasicInfo(round, index) {
  const stats = getRoundStatistics(round);
  return (
      <div key={'round' + round.id + index} className="observation-round-overview">
        <div className="header-1">
          Tilsynsrunde {getDateAsString(round.createdTimestamp)}
        </div>
        <Map observationRound={round} style={{height: '300px', width: '100%', border: '1px solid black'}} bounds={getMapBoundsForEntireRound(round)}/>
        <div className="header-3">Oppsummering:</div>
        <div className="round-metadata">
          <StatHolder svg={true} bigger={true} icon={EweImage} count={getNumberOfSheepInObservationRound(round).ewes} statName="Søyer" />
          <StatHolder svg={true} bigger={false} icon={LambImage} count={getNumberOfSheepInObservationRound(round).lamb} statName="Lam" />
          <StatHolder svg={true} bigger={true} icon={InjuredImage} count={stats.injuryCount} statName="Skadd sau" />
          <StatHolder svg={true} bigger={true} icon={DeadImage} count={stats.deadCount} statName="Død sau" />
          <StatHolder svg={true} bigger={true} icon={WoolImage} count={stats.woolCount} statName="Ullfunn" />
        </div>
        <div className="round-metadata">
          <StatHolder svg={true} bigger={true} icon={PredatorImage} count={stats.predatorCount} statName="Rovdyr" />
          <StatHolder svg={true} bigger={true} icon={DogImage} count={stats.dogCount} statName="Hund" />
          <StatHolder svg={false} bigger={false} icon="fa-clock" count={getTimeSpent(round)} statName="" />
          <StatHolder svg={false} bigger={false} icon="fa-blind" count={getWalkedDistance(round)} statName="" />
        </div>
      </div>
  );
}

function observationHeader(count, data) {
  let icon = '';
  let text = '';
  switch (data.type) {
    case 'Sau': {
      if (data.category === 'Søye alene') {
        icon = <img className="bigger" src={EweImage} alt="Søye alene" />;
        text = 'Søye alene';
      } else if(data.category === 'Lam alene') {
        icon = <img src={LambImage} alt="Lam alene" />;
        text = 'Lam alene';
      } else {
        icon = <img className="bigger" src={SheepImage} alt="Sau" />;
        text = 'Gruppe med sau';
      }
      break;
    }
    case 'Død sau': {
      icon = <img className="bigger" src={DeadImage} alt="Død sau" />;
      text = 'Død sau';
      break;
    }
    case 'Ullfunn': {
      icon = <img className="bigger" src={WoolImage} alt="Ullfunn" />;
      text = 'Ullfunn';
      break;
    }
    case 'Hund': {
      icon = <img className="bigger" src={DogImage} alt="Hund" />;
      text = data.category;
      break;
    }
    case 'Rovdyr': {
      icon = <img className="bigger" src={PredatorImage} alt="Rovdyr" />;
      text = data.category;
      break;
    }
    case 'Reinsdyr': {
      icon = <img className="bigger" src={ReindeerImage} alt="Reinsdyr" />;
      text = 'Reinsdyr';
      break;
    }
    case 'Annet': {
      icon = <img className="bigger" src={OtherImage} alt="Annet" />;
      text = 'Annet';
      break;
    }
    default: {
      icon = <img className="bigger" src={OtherImage} alt="Annet" />;
      text = 'Ukjent';
      break;
    }
  }
  return (
      <div className="observation-header">
        <div className="report-header-icon-container">
          {count}
        </div>
        <div className="text">
          {text}
        </div>
      </div>
  )
}

export function observationRoundDetailedInfo(round) {
  let observationCount = 0;
  const output = [];
  Object.keys(round.registrationPoints).forEach((key, index) => {
    const registrationPoint = round.registrationPoints[key];
    const observations = Object.keys(registrationPoint.observations).map(key => {
      const observation = registrationPoint.observations[key];
      observationCount += 1;
      return (
          <div key={'observation' + observation.timestamp + index} className="observation-overview">
            {observationHeader(observationCount, observation.data)}
            <Map observationRound={round} style={{height: '300px', width: '100%', border: '1px solid black'}} bounds={getBoundsForObservation(registrationPoint, observation)}/>
            {observationRoundContent(observation.data)}
          </div>
      )
    });
    output.push(
        <div key={'observations'+ round.id + index} className="observations">
          {observations}
        </div>
    );
  });
  return output;
}

function getInjuries(data, single) {
  if (single) {
    return (
        <div className="inury-container">
          <div className="group">
            <div className="header-3">Skade:</div>
            <DataRow name="Skade:" value={data.injuryCategory} />
            <DataRow name="Hvor:" value={data.injury} />
            <DataRow name="Hva ble gjort med sauen?" value={data.decision} />
            <DataRow name="Hvem ble kontaktet angående skadd sau?" value={data.contacted === null ? 'Ingen' : data.contacted} />
          </div>
        </div>
    )
  }
  return (
      <div className="inury-container">
        <div className="group">
          <div>
            <DataRow name="Antall skadde?" value={data.injuries.length} />
          </div>
          {
            data.injuries.map((injury, index) => {
              return (
                  <div key={data + index}>
                    <div className="header-3">
                      Nr. {index + 1}
                    </div>
                    <DataRow name="Skade:" value={injury.injuryCategory} />
                    <DataRow name="Hvor:" value={injury.injury} />
                    <DataRow name="Type sau:" value={injury.sheepType} />
                    <DataRow name="Farge:" value={injury.color} />
                    <DataRow name="Hva ble gjort med sauen?" value={injury.decision} />
                  </div>
              )
            })
          }
          <div className="header-3">Varsling:</div>
          <div className="group">
            <DataRow name="Hvem ble kontaktet angående skadd sau?" value={data.contacted === null ? 'Ingen' : data.contacted} />
          </div>
        </div>
      </div>
  );
}

export function observationRoundContent(data) {
  let content = '';
  let images = '';
  let description = '';
  if (data.images.length > 0) {
    images = (
        <div className="group">
          <div className="header-2">Bilder:</div>
          <div className="images">
            {data.images.map(image => <img className="report-image" key={image} src={image} />)}
          </div>
        </div>
    );
  }
  if (data.description !== null) {
    description = (
        <div className="metadata">
          <div className="header-3">Beskrivelse:</div>
          <div className="description-box">
            {data.description}
          </div>
        </div>
    )
  }

  switch (data.type) {
    case 'Sau': {
      let injury = null;
      if (data.category === 'Søye alene') {
        if (data.hasInjury === 'Ja') {
          injury = getInjuries(data, true);
        }
        content = (
          <div className="metadata">
            <div className="header-2">Informasjon:</div>
            <div className="group">
              <DataRow name="Farge:" value={getSheepColor(data.groups[0])} />
              <DataRow name="Har skade?" value={data.hasInjury} />
            </div>
            {injury}
          </div>
        );
      } else if(data.category === 'Lam alene') {
        if (data.hasInjury === 'Ja') {
          injury = getInjuries(data, true);
        }
        content = (
            <div className="metadata">
              <div className="header-2">Informasjon:</div>
              <div className="group">
                <DataRow name="Farge:" value={getSheepColor(data.groups[0])} />
                <DataRow name="Har skade?" value={data.hasInjury} />
              </div>
              {injury}
            </div>
        );
      } else {
        if (data.hasInjury === 'Ja') {
          injury = getInjuries(data, false);
        }
        const antall = getNumberOfSheepInObservation(data);
        console.log(antall);
        content = (
            <div className="metadata">
              <div className="header-2">Informasjon:</div>
              <div className="header-3 no-top-padding">Søyer:</div>
              <div className="group">
                <DataRow name="Antall hvite" value={antall.ewes.white} />
                <DataRow name="Antall svarte" value={antall.ewes.black} />
                <DataRow name="Antall hvite og svarte" value={antall.ewes.blackAndWhite} />
              </div>

              <div className="header-3">Lam:</div>
              <div className="group">
                <DataRow name="Antall hvite" value={antall.lamb.white} />
                <DataRow name="Antall svarte" value={antall.lamb.black} />
                <DataRow name="Antall hvite og svarte" value={antall.lamb.blackAndWhite} />
              </div>

              <div className="header-3">Skadd sau:</div>
              <div className="group">
                <DataRow name="Inneholder gruppen skadd sau?" value={data.hasInjury} />
                {injury}
              </div>
            </div>
        );
      }
      break;
    }
    case 'Død sau': {
      content = (
          <div className="metadata">
            <div className="header-2">Informasjon:</div>
            <DataRow name="Type sau:" value={data.sheepType}/>
            <DataRow name="Farge:" value={data.color}/>
            <DataRow name="Dødsårsak:" value={data.causeOfDeath}/>
            <DataRow name="Dødstidspunkt:" value={data.timeOfDeath}/>
            <DataRow name="Har sauen blitt tildekt?" value={data.covered}/>
            <DataRow name="Hvem har blitt kontaktet angående den døde sauen?" value={data.contacted === null ? 'Ingen' : data.contacted}/>
          </div>
      );
      break;
    }
    case 'Ullfunn': {
      content = (
          <div className="metadata">
            <div className="header-2">Informasjon:</div>
            <DataRow name="Farge på ull:" value={data.color}/>
          </div>
      );
      break;
    }
    case 'Hund': {
      const dogInfo = (
          <div className="group">
            <div className="header-3 no-top-padding">Informasjon om hund:</div>
            <DataRow name="Farge:" value={data.dogColor}/>
            <DataRow name="Hunderase:" value={data.breed}/>
            <DataRow name="Pelstype:" value={data.furType}/>
            <DataRow name="Halsbånd:" value={data.collar}/>
          </div>
      );
      const ownerInfo = (
          <div className="group">
            <div className="header-3">Informasjon om eier:</div>
            <DataRow name="Navn på eier:" value={data.ownerName === null ? 'Ukjent' : data.ownerName}/>
            <DataRow name="Kjønn på eier:" value={data.ownerGender}/>
            <DataRow name="Alder på eier:" value={data.ownerAge}/>
            <DataRow name="Farge på bukse:" value={data.pantsColor}/>
            <DataRow name="Farge på overdel:" value={data.jacketColor}/>
            <DataRow name="Hårfarge:" value={data.ownerHairColor}/>
            <DataRow name="Bar eieren våpen?" value={data.gun}/>
            <DataRow name="Var det med flere personer?" value={data.otherPersons}/>
            <DataRow name="Størrelse på sekk:" value={data.backpackSize}/>
          </div>
      );
      if (data.category === 'Hund med eier') {
        content = (
            <div className="metadata">
              <div className="header-2">Informasjon:</div>
              {dogInfo}
              {ownerInfo}
            </div>
        );
      } else {
        content = (
            <div className="metadata">
              <div className="header-2">Informasjon:</div>
              {dogInfo}
            </div>
        )
      }
      break;
    }
    case 'Rovdyr': {
      content = (
          <div className="metadata">
            <div className="header-2">Informasjon:</div>
            <DataRow name="Dyr:" value={data.animal}/>
          </div>
      );
      break;
    }
    case 'Reinsdyr': {
      content = (
          <div className="metadata">
            <div className="header-2">Informasjon:</div>
            <DataRow name="Antall:" value={data.count}/>
          </div>
      );
      break;
    }
    case 'Annet': {
      content = (
          <div className="metadata">
            <div className="header-3">Ingen informasjon:</div>
          </div>
      );
      break;
    }
    default: {
      content = (
          <div className="metadata">
            <div className="header-3">Ingen informasjon:</div>
          </div>
      );
      break;
    }
  }

  return (
      <div className="observation-content">
        {content}
        {description}
        {images}
      </div>
  )
}

